// BEGIN CMVC
// ************************************************************************
//
//   Workfile: IncrementEJBPea.java, v3-samples, fmv3
//   Last update: 01/03/23 13:42:45
//   SCCS path, id: /home/flowmark/vc/0/3/7/8/s.70 1.1
//
// ************************************************************************
// END CMVC
// BEGIN COPYRIGHT
// ************************************************************************
//
//   Licensed Materials - Property of IBM
//   WA03 SupportPac
//   (c) Copyright IBM Corp. 2001
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract
//   with IBM Corp.
//
// ************************************************************************
// END COPYRIGHT
// BEGIN DISCLAIMER
// ************************************************************************
//   DISCLAIMER
//   ----------
//   This material contains programming source code for your consideration.
//   These examples have not been thoroughly tested under all conditions.
//   IBM, therefore, cannot guarantee or imply reliability, serviceability,
//   or function of these programs.
//   ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//   WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//   NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//   FOR A PARTICULAR PURPOSE.
// ************************************************************************
// END DISCLAIMER

import java.io.*;
import java.util.*;
import javax.naming.*;
import javax.ejb.*;
import com.ibm.workflow.api.*;
import com.transarc.jmon.examples.Inc.*;

/**
 * <p>
 * MQSeries Workflow EJB invocation sample
 * </p><p>
 * This sample shows you how to invoke an EJB using the PEA and the Java
 * High Performance Bridge
 * </p><p>
 * This way to invoke EJBs allows you to retrieve the Workflow containers metamodel
 * but does not support transactions
 * </p><p>
 * The High Performance Bridge caches the static variables of this class
 * this is a major advantage, but must be used with caution.
 * <ul>
 *    <li>You can cache variables, that are used in common from all instances of this class,
 *        like the JNDI context or EJB homes. These variables don't change and are the same
 *        for every instance of this class. They can be treated like constants and you don't
 *        have to synchronize the access on them.</li>
 *    <li>When using static variables that may change during the lifetime of a PEA session,
 *        like a counter for the number of object instances of this class,
 *        make sure that you synchronize the access on them.</li>
 *    <li>You should not cache variables like EJB primary keys, that are usually different 
 *        in every object of the class.</li>
 * </ul></p>
 */
public class IncrementEJBPea
{
  /** JNDI naming context */
  static private javax.naming.Context          jndiContext    = null; 
  /** EJB home interface */
  static private IncHome                       incrementHome  = null; 
  /** MQWF embedded Agent */
  static private Agent                         agent          = null;
  /** MQWF Execution Agent, needed for workflow container access */
  static private ExecutionAgent                exeAgent       = null;
  /** Command to be executed in this invocation */
         private String                        command        = null;
  /** Workflow process instance name */
         private String                        instanceName   = null; 





  /**
   * Dispatcher method that actually invokes the EJB methods
   * 
   * @return The value of the increment counter in the EJB
   * 
   * @exception Exception Any occurred exception will be rethrown unfiltered
   *
   */
  private int executeCommand() throws Exception
  {
    long     startMillis;  // For performance measuring
    long     stopMillis;
    IncKey   incrementKey; // EJB primary key
    Inc      increment;    // EJB remote interface


    // Create the primary key of the EJB, the actual key value is the process instance name
    startMillis = System.currentTimeMillis();
    incrementKey = new IncKey(instanceName);
    stopMillis = System.currentTimeMillis();
    System.out.println("Time consumed for creating the primary Key (ms): " + (stopMillis - startMillis));


    // Handle the EJB create request
    startMillis = System.currentTimeMillis();
    if(command.equalsIgnoreCase("create"))
    {
      increment = incrementHome.create(incrementKey); // Actual EJB invocation
    }
    
    // Handle the EJB increment request
    if(command.equalsIgnoreCase("incrementandgetvalue"))
    {
      increment = incrementHome.findByPrimaryKey(incrementKey);  // Retrieve the EJB remote interface
      increment.increment();                                     // Actual EJB invocation
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for executing the command " + command + " (ms): " + (stopMillis - startMillis));
      return increment.getValue();                               // Actual EJB invocation
    }
    
    // Handle the EJB remove request
    if(command.equalsIgnoreCase("remove"))
    {
      increment = incrementHome.findByPrimaryKey(incrementKey);  // Retrieve the EJB remote interface 
      increment.remove();                                        // Actual EJB invocation
    }
    stopMillis = System.currentTimeMillis();
    System.out.println("Time consumed for executing the command " + command + " (ms): " + (stopMillis - startMillis));
    
    return 0;
  } // private int executeCommand() throws Exception
  




  /**
   * Dummy constructor
   */
  public IncrementEJBPea()
  {
    super();
  }





  /**
   * Main method, no parameter evaluation
   */
  public static void main(String[] args)
  {
    long                                    startMillis;  // For performance measuring
    long                                    stopMillis;
    int                                     counterValue; // Just for information transport
    IncrementEJBPea                         myself;       // Actual instance of this class


    // Note that console output in the PEA window is a performance killer, you should
    // use it only for debugging purposes, not for production.

    // Show the classpath in the PEA console
    System.out.println("Classpath: " + System.getProperty("java.class.path"));

    // Create the actual object instance of this class
    myself = new IncrementEJBPea();

    try
    {
      // Create the Agent and ExecutionAgent. As this variable is cached, this code runs only once
      startMillis = System.currentTimeMillis();
      if(agent == null)
      {
        agent = new Agent();
        agent.setLocator(Agent.LOC_LOCATOR); // Container access is possible only with LOC_LOCATOR
        agent.setName("NONE");               // Dummy name for LOC_LOCATOR
        exeAgent = agent.getExecutionAgent();
      }
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for initializing the Agent (ms): " + (stopMillis - startMillis));


      // Create the JNDI naming context. As this variable is cached, this code runs only once
      startMillis = System.currentTimeMillis();
      if(jndiContext == null)
      {
        Properties prop = new Properties(); // Temporary needed properties

        // Read properties from configuration file
        prop.load(new FileInputStream("EJBInvocation.properties")); 
        // Translate property names
        prop.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, prop.getProperty("EJB.INITIAL_CONTEXT_FACTORY"));
        prop.put(javax.naming.Context.PROVIDER_URL,            prop.getProperty("EJB.PROVIDER_URL"));

        // Create the context
        jndiContext = new javax.naming.InitialContext(prop);
      }
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for getting the initial JNDI context (ms): " + (stopMillis - startMillis));


      // Retrieve the EJB home. As this variable is cached, this code runs only once
      startMillis = System.currentTimeMillis();
      if(incrementHome == null)
      {
        Object obj; // Temporary needed dummy object

        obj = jndiContext.lookup("IncBean");
        System.out.println(obj.toString()); // See what comes, just for debugging purposes
        incrementHome = (IncHome) javax.rmi.PortableRemoteObject.narrow(obj, IncHome.class);
      }
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for retrieving the EJB Home (ms): " + (stopMillis - startMillis));
      
      
      // Read the input container
      startMillis = System.currentTimeMillis();
      myself.readContainer();
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for reading the input container (ms): " + (stopMillis - startMillis));


      // Execute the command retrieved from the input container
      counterValue = myself.executeCommand();
      System.out.println("New counter value: " + counterValue);


      // Write the counter value to the output container
      startMillis = System.currentTimeMillis();
      myself.writeContainer(counterValue);
      stopMillis = System.currentTimeMillis();
      System.out.println("Time consumed for writing the output container (ms): " + (stopMillis - startMillis));
    }
    // Catch all exceptions here. Please don't treat this code as a sample for how to handle exceptions
    catch (Throwable exc) 
    {
      System.out.println(exc.getMessage());
      exc.printStackTrace();
      try
      {
        Thread.sleep(10000);
      }
      catch (java.lang.InterruptedException e){}
    }
  } // public static void main(String[] args)





  /**
   * Reads all needed variables from the input container and stores the values 
   * in the class variables
   * 
   * @exception com.ibm.workflow.api.FmcException Any occurred exception will be rethrown
   */
  private void readContainer() throws com.ibm.workflow.api.FmcException
  {
    ReadOnlyContainer roContainer = exeAgent.inContainer();

    command = roContainer.getString("Command");
    instanceName = roContainer.getString("_PROCESS");
  }


  


  /**
   * Writes the increment counter value to the output container
   * 
   * @param count The value of the increment counter in the EJB
   * 
   * @exception com.ibm.workflow.api.FmcException Any occurred exception will be rethrown
   *
   */
  private void writeContainer(int count) throws com.ibm.workflow.api.FmcException
  {
    ReadWriteContainer rwContainer = exeAgent.outContainer();

    rwContainer.setLong("CounterValue", count);
    exeAgent.setOutContainer(rwContainer);
  }
}
